﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SCMFileContentSearch
{
    class QueryContents
    {
        static bool IsTextFile(string fileName)
        {
            byte[] file;
            System.IO.FileStream stream = new System.IO.FileStream(fileName, System.IO.FileMode.Open, System.IO.FileAccess.Read);
            if (stream.Length < 10485760)
            {
                using (stream)
                {

                    file = new byte[stream.Length];
                    stream.Read(file, 0, file.Length);

                }

                if (file.Length > 3 && ((file[0] == 0x00 && file[1] == 0x00 && file[2] == 0xFE && file[2] == 0xFF /*UCS-4*/)))
                    return true;
                else if (file.Length > 2 && ((file[0] == 0xEF && file[1] == 0xBB && file[2] == 0xBF /*UTF-8*/)))
                    return true;
                else if (file.Length > 1 && ((file[0] == 0xFF && file[1] == 0xFE /*Unicode*/)))
                    return true;
                else if (file.Length > 1 && (file[0] == 0xFE && file[1] == 0xFF /*Unicode Big Endian*/))
                    return true;
                else
                {
                    for (int i = 0; i < file.Length; i++)
                        if (file[i] > 0x80)
                            return false;

                    return true;
                }
            }
            else
            {
                return false;
            }

        }
        public IEnumerable<string> Query(string startFolder, string searchTerm)
        {
            // Modify this path as necessary.
            //string startFolder = @"c:\program files\Microsoft Visual Studio 9.0\";

            // Take a snapshot of the file system.
            IEnumerable<System.IO.FileInfo> fileList = GetFiles(startFolder);

            //string searchTerm = @"Private m_Dirty";

            // Search the contents of each file.
            // A regular expression created with the RegEx class
            // could be used instead of the Contains method.
            // queryMatchingFiles is an IEnumerable<string>.
            var queryMatchingFiles =
                from file in fileList
                //where file.Extension == ".htm"
                let fileText = GetFileText(file.FullName)
                where fileText.Contains(searchTerm)
                select file.FullName;
            return queryMatchingFiles;
            // Execute the query.
            //Console.WriteLine("The term \"{0}\" was found in:", searchTerm);
            //foreach (string filename in queryMatchingFiles)
            //{
            //    Console.WriteLine(filename);
           // }

            // Keep the console window open in debug mode.
           // Console.WriteLine("Press any key to exit");
           // Console.ReadKey();
        }

        // Read the contents of the file.
        static string GetFileText(string name)
        {
            string fileContents = String.Empty;

            // If the file has been deleted since we took 
            // the snapshot, ignore it and return the empty string.
            
            if ((System.IO.File.Exists(name))&&(IsTextFile(name)))
            {
                fileContents = System.IO.File.ReadAllText(name);
            }
            return fileContents;
        }

        // This method assumes that the application has discovery 
        // permissions for all folders under the specified path.
        static IEnumerable<System.IO.FileInfo> GetFiles(string path)
        {
            if (!System.IO.Directory.Exists(path))
                throw new System.IO.DirectoryNotFoundException();

            string[] fileNames = null;
            List<System.IO.FileInfo> files = new List<System.IO.FileInfo>();

            fileNames = System.IO.Directory.GetFiles(path, "*.*", System.IO.SearchOption.AllDirectories);
            foreach (string name in fileNames)
            {
                files.Add(new System.IO.FileInfo(name));
            }
            return files;
        }
    }

}
