﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Management;
using System.Diagnostics;

namespace SCMFileContentSearch
{
    public partial class frmMain : Form
    {
        public SCMConfig config;
        public string scmServerAddress;
        public string scmServerPort;
        public string scmUserName;
        public string scmPassword;
        public string scmTempAddress;

        public string appPath = Application.StartupPath;

        public frmMain()
        {
            InitializeComponent();
            config = new SCMConfig(appPath);
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            string scmCommand = "";
            string result = "";
            string repoPath = "";
            //First we need to strip out the branch type out of the name
            string branch = cmbBranch.Text;
            if ((branch.Contains(")")) || (branch.Contains("(")))
            {
                branch = branch.Substring(0, branch.LastIndexOf(")") - 1);
                branch = branch.Substring(0, branch.LastIndexOf("(") - 1);
            }

            //Need to see if we are searching the mainline or child branch...
            if (cmbMainline.Text != branch)
            {
                //if the branch is not the mainline be need to get the repo path for the root.
                scmCommand = "branchproperty  -b\"" + branch + "\"" + " -p\"" + cmbMainline.Text + "\"" + " -y" + scmUserName + ":" + scmPassword + " -z" + scmServerAddress + ":" + scmServerPort;
                result = sscmexe.sscmCMD(scmCommand);
                if (result.StartsWith("Error:"))
                {
                    MessageBox.Show("Error getting branch properties.\n" + result + "\n" + "Please check your branch selection and settings.", "Find in Files - Surround SCM");
                    
                }
                else
                {
                    
                    repoPath = result.Substring(result.IndexOf("\n") + 1);
                    repoPath = repoPath.Substring(repoPath.IndexOf("\n") + 1);
                    int colPos = repoPath.IndexOf(":") + 1;
                    int subLenght = repoPath.IndexOf("\r") - colPos;
                    repoPath = repoPath.Substring(colPos, subLenght);
                    repoPath = repoPath.Trim();
                }

            }
            else
            {
                repoPath = cmbMainline.Text;
            }
            if (repoPath != "")
            {
                string path = "";
                if (scmTempAddress.EndsWith("\\"))
                {
                    path = scmTempAddress + "SCMTemp";
                }
                else
                {
                    path = scmTempAddress + "\\" + "SCMTemp";
                }

                if (System.IO.Directory.Exists(path))
                {
                    System.IO.DirectoryInfo dir = new System.IO.DirectoryInfo(path);
                    dir.Attributes = System.IO.FileAttributes.Normal;
                    dir.Delete(true);  // delete recursively

                }

                System.IO.Directory.CreateDirectory(path);

                lstFiles.Items.Clear();

                lstFiles.Items.Add("Starting process at " + DateTime.Now.ToString());

                scmCommand = "get / -d\"" + path + "\"" + " -b\"" + branch + "\"" + " -p\"" + repoPath + "\"" + " -e -r -y" + scmUserName + ":" + scmPassword + " -z" + scmServerAddress + ":" + scmServerPort;
                result = sscmexe.sscmCMD_Call(scmCommand);
                if (result != null)
                {
                    lstFiles.Items.Add(result);
                }

                QueryContents qc = new QueryContents();
                IEnumerable<string> queryMatchingFiles = qc.Query(path, @"" + txtFind.Text);

                lstFiles.Items.Add("Searching for " + txtFind.Text + "...");
                foreach (string filename in queryMatchingFiles)
                {
                    string scmFileName = filename.Substring(filename.IndexOf(path) + path.Length + 1);
                    lstFiles.Items.Add("sscm://" + scmServerAddress + ":" + scmServerPort + "//" + branch + "//" + repoPath + "/" + scmFileName.Replace("\\", "/"));

                }
                lstFiles.Items.Add("Finished process at " + DateTime.Now.ToString());
            }
            Cursor = Cursors.Default;
        }


        private void frmMain_Load(object sender, EventArgs e)
        {
            if (config.Exists())
            {
                try
                {
                    //Load settings from file
                    scmServerAddress = config.getServerAddress();
                    scmServerPort = config.getServerPort();
                    scmUserName = config.getSCMUserName();
                    scmPassword = config.getSCMPassword();
                    scmTempAddress = config.getTempDir();
                    //Populate fields in settings tab
                    txtDir.Text = scmTempAddress;
                    txtSCMServerAdd.Text = scmServerAddress;
                    txtSCMSrvPort.Text = scmServerPort;
                    txtSCMUser.Text = scmUserName;
                    txtSCMPassword.Text = scmPassword;
                    //Load mainline list
                    LoadMainlines();
                    
                }
                catch
                {
                    tabControl1.SelectedIndex = 1;
                    lblStatus.Visible = true;
                    lblStatus.Text = "A problem was encountered when trying to retrieve the list of mainline branches.  Please check your connection settings.";
                    this.Height = 468;
                    tabControl1.Height = 440;

                    
                }
            }
            else
            {
                tabControl1.SelectedIndex = 1;
                lblStatus.Visible = true;
                lblStatus.Text = "A configuration file was not found.  Please fill out the information above and click 'Save' to create a configuration file.";
                this.Height = 468;
                tabControl1.Height = 440;
            }
        }

        private void txtSave_Click(object sender, EventArgs e)
        {
            if ((txtDir.Text == "")||(txtSCMServerAdd.Text =="")||(txtSCMSrvPort.Text == "")||(txtSCMUser.Text ==""))
            {
                MessageBox.Show("Required information missing!  Please check your settings","Find in Files - Surround SCM");
            }
            else
            {
                string result = config.SaveConfigFile(txtDir.Text, txtSCMServerAdd.Text, txtSCMSrvPort.Text, txtSCMUser.Text, txtSCMPassword.Text);
                if (result == "yes")
                {

                    //Notify user of success saving conf file
                    MessageBox.Show("Configuration File created!", "Find in Files - Surround SCM");

                    //We are going to reconnect with new saved settings
                    //Clear combo boxes that might have been populated with prior connection
                    cmbMainline.Items.Clear();
                    cmbBranch.Items.Clear();
                    cmbBranch.Text = "";

                    //Reset variables based on new values in text fields
                    scmServerAddress = txtSCMServerAdd.Text;
                    scmServerPort = txtSCMSrvPort.Text;
                    scmUserName = txtSCMUser.Text;
                    scmPassword = txtSCMPassword.Text;
                    scmTempAddress = txtDir.Text;

                    //Make sure error label is hidden 
                    lblStatus.Text = "";
                    lblStatus.Visible = false;
                    this.Height = 388;
                    tabControl1.Height = 360;

                    //Start process of reloading mainline branch combo box
                    LoadMainlines();
                                                           
                }
                else
                {
                    MessageBox.Show("Error Encountered:\n " + result,"Find in Files - Surround SCM");
                }
            }

        }

        

        private void cmbMainline_SelectedIndexChanged(object sender, EventArgs e)
        {
            //New mainline is selected, so let's re-load the branch list
            cmbBranch.Text = "<Populating branch list...>";
            cmbBranch.Items.Clear();
            string mainline = cmbMainline.Text;
            string scmCommand = "lsbranch  -p\""+ mainline+ "\" -y" + scmUserName + ":" + scmPassword + " -z" + scmServerAddress + ":" + scmServerPort;
            string result = sscmexe.sscmCMD(scmCommand);

            //let's process the result and add the mainelines
            if (result.StartsWith("Error:"))
            {
                MessageBox.Show("Error loading list of branches.\n" + result + "\n" + "Please check your settings.", "Find in Files - Surround SCM");
            }
            else
            {
                int pos = result.IndexOf("\n");
                while (pos > 1)
                {
                    cmbBranch.Items.Add(result.Substring(0, pos - 2));
                    result = result.Substring(pos + 1);
                    pos = result.IndexOf("\n");

                }
                cmbBranch.Text = "<Select Branch>";
            }
        }

        private void btnGoTo_Click(object sender, EventArgs e)
        {
            GoToFile();
            
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            dirBrowser.ShowDialog();
            txtDir.Text = dirBrowser.SelectedPath;
            
        }
        private void GoToFile()
        {
            try
            {
                Process sscm = new Process();
                sscm.StartInfo.FileName = "C:\\Program Files\\Seapine\\Surround SCM\\Surround SCM Client.exe";
                sscm.StartInfo.Arguments = "\"" + lstFiles.Text + "\"";
                //sscm.StartInfo.CreateNoWindow = false;
                sscm.Start();
            }
            catch (Exception f)
            {
                MessageBox.Show("Error: " + f.Message ,"Find in Files - Surround SCM");
            }
        }

        private void lstFiles_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            GoToFile();
        }
        private void LoadMainlines()
        {
            string scmCommand = "lsmainline -y" + scmUserName + ":" + scmPassword + " -z" + scmServerAddress + ":" + scmServerPort;
            string result = sscmexe.sscmCMD(scmCommand);

            //let's process the result and add the mainelines
            if (result.StartsWith("Error:"))
            {
                MessageBox.Show("Error loading list of mainline branches.\n" + result + "\n" + "Please check your settings.","Find in Files - Surround SCM");
                tabControl1.SelectedIndex = 1;
            }
            else
            {
                int pos = result.IndexOf("\n");
                while (pos > 1)
                {
                    cmbMainline.Items.Add(result.Substring(0, pos - 2));
                    result = result.Substring(pos + 1);
                    pos = result.IndexOf("\n");
                }
                if (cmbMainline.Items.Count > 0)
                {
                    cmbMainline.Text = "<Select Mainline>";
                }
                tabControl1.SelectedIndex = 0;
            }
        }

        

       

        
    }
}
